@@ -83,27 +83,21 @@
 //!# ToDo List
 //! Unify GenTransaction and WebTransaction. File upload doesn't work with WebTransaction currently.
 //!
-//! Implement ALTER TABLE, RENAME TABLE.
-//! Design of ALTER TABLE : want to preserve Table Id and all Field Ids.
-//!
-//! Make copy of existing Table Info, adjust it, based on AlterActions.
-//! Copy all reords to new file.
-//! Add indexes.
-//!
-//! Combine SortedFile pages after deletes ( either at once or later ).
-//!
 //! Implement email in example program. Replication of log files. Server status.
 //!
 //! Sort out error handling for PARSEINT etc.
 //!
 //! Work on improving/testing SQL code, browse schema, float I/O. Login.
+//!
+//! If updating several databases atomically, want to be able to share the upd file.
+//! So have mechanism to put updates to several files in one upd file.
 
 pub use crate::{
     atomfile::AtomicFile,
     gentrans::{GenTransaction, Part},
     init::INITSQL,
     pstore::{AccessPagedData, SharedPagedData},
-    stg::{MemFile, SimpleFileStorage},
+    stg::{MemFile, SimpleFileStorage, Storage},
     webtrans::WebTransaction,
 };
 
@@ -134,7 +128,6 @@ use crate::{
     parse::Parser,
     run::*,
     sortedfile::{Asc, Id, Record, SortedFile},
-    stg::Storage,
     table::{ColInfo, IndexInfo, Row, SaveOp, Table, TablePtr},
     util::{nd, newmap, SmallSet},
     value::*,
@@ -458,6 +451,25 @@ GO
         }
     }
 
+    pub(crate) fn repack_file(self: &DB, k: i64, schema: &str, tname: &str) -> i64 {
+        if k >= 0 {
+            let name = ObjRef::new(schema, tname);
+            if let Some(t) = self.get_table(&name) {
+                return t.repack(self, k as usize);
+            }
+        } else {
+            let k = (-k - 1) as usize;
+            if k < 4 {
+                return self.bs[k].repack_file(self);
+            }
+        }
+        -1
+    }
+
+    pub(crate) fn page_size(&self, pnum: u64) -> u64 {
+        self.file.spd.file.read().unwrap().page_size(pnum) as u64
+    }
+
     /// Save updated tables to underlying file ( or rollback if there was an error ).
     /// Returns the number of logical pages that were updated.
     pub fn save(self: &DB) -> usize {
