@@ -21,7 +21,7 @@ pub struct Table {
     /// Type information about the columns.
     pub info: Rc<ColInfo>,
 
-    /// List of indexes.
+    /// List of indexes. ( Maybe could eliminate the RefCell )
     pub ixlist: RefCell<IxList>,
 
     /// Table id in sys.Table.
@@ -90,6 +90,22 @@ impl Table {
         row.delcodes(db); // Deletes codes for Binary and String values.
     }
 
+    pub fn repack(&self, db: &DB, k: usize) -> i64 {
+        let row = self.row();
+        if k == 0 {
+            self.file.repack(db, &row)
+        } else {
+            let list = &*self.ixlist.borrow();
+            if k <= list.len() {
+                let (f, cols) = &list[k - 1];
+                let ixr = IndexRow::new(self, cols.clone(), &row);
+                f.repack(db, &ixr)
+            } else {
+                -1
+            }
+        }
+    }
+
     /// Look for indexed table expression based on supplied WHERE expression (we).
     pub fn index_from(
         self: &TablePtr,
